#include <string.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/sys/printk.h>

#include <zephyr/drivers/uart.h>

#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(uart_log, LOG_LEVEL_DBG);

// 1000 msec = 1 sec
#define SLEEP_TIME_MS   1000

//the size of the receive buffer 
#define RECEIVE_BUFF_SIZE 40

//the receiving timeout period 
#define RECEIVE_TIMEOUT 100

int ret;


// Get the device pointer of the UART hardware 
const struct device *uart= DEVICE_DT_GET(DT_NODELABEL(uart1));

// Define the transmission buffer, which is a buffer to hold the data to be sent over UART */
static uint8_t tx_buf[] =   {"Hi, some data was sent.\n\r"};


#define LED0_NODE DT_ALIAS(led0)
#define LED1_NODE DT_ALIAS(led1)

static const struct gpio_dt_spec led0 = GPIO_DT_SPEC_GET(LED0_NODE, gpios);
static const struct gpio_dt_spec led1 = GPIO_DT_SPEC_GET(LED1_NODE, gpios);

// SW0_NODE is the devicetree node identifier for the "sw0" alias 
#define SW0_NODE	DT_ALIAS(sw0) 
static const struct gpio_dt_spec button = GPIO_DT_SPEC_GET(SW0_NODE, gpios);

// Define the receive buffer 
static uint8_t rx_buf[RECEIVE_BUFF_SIZE] = {0};

// Define the callback function
void button_pressed(const struct device *dev, struct gpio_callback *cb, uint32_t pins)
{
  
	int ret;
	ret = uart_tx(uart, tx_buf, sizeof(tx_buf), SYS_FOREVER_MS);
	if (ret) {
		LOG_INF("Error callback button uart tx!");
		return;
	}
}
//Define a variable of type static struct gpio_callback 
static struct gpio_callback button_cb_data;


#pragma GCC push_options
//#pragma GCC optimize ("O0")
static void uart_cb(const struct device *dev, struct uart_event *evt, void *user_data)
{

	switch (evt->type) {

	case UART_RX_RDY:

		LOG_INF("%s",rx_buf);
		//memcpy(&rx_buf[0], 0, RECEIVE_BUFF_SIZE); // set to zero.
		for(size_t i = 0;i<RECEIVE_BUFF_SIZE;i++)
		{
			rx_buf[i] = 0; 
		}

		gpio_pin_toggle_dt(&led1);
		break;

	case UART_RX_DISABLED:
		uart_rx_enable(dev ,rx_buf,sizeof rx_buf,RECEIVE_TIMEOUT);
		break;
		
	default:
		break;
	}
}
#pragma GCC pop_options

int main(void)
{
	int ret;

	LOG_INF("Lets start!");
	if (!device_is_ready(uart)){
		
		return 1 ;
	}

	if (!device_is_ready(button.port)) {
		LOG_INF("Button not initialized.\r\n");
		return 1 ;
	}

	ret = gpio_pin_configure_dt(&led0, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		LOG_INF("led 0 not initialized.\r\n");
		return 1; 
	}
	
	ret = gpio_pin_configure_dt(&led1, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		LOG_INF("led 1 not initialized.\r\n");
		return 1; 
	}

	ret = gpio_pin_configure_dt(&button, GPIO_INPUT);
	if (ret < 0) {
		return 1;
	}

	//Configure the interrupt on the button's pin
	ret = gpio_pin_interrupt_configure_dt(&button, GPIO_INT_EDGE_TO_ACTIVE );

	//  Initialize the static struct gpio_callback variable   
    gpio_init_callback(&button_cb_data, button_pressed, BIT(button.pin)); 	
	
	// Add the callback function by calling gpio_add_callback()   
	 gpio_add_callback(button.port, &button_cb_data);

	// Register the UART callback function 
	ret = uart_callback_set(uart, uart_cb, NULL);
	if (ret) {
		LOG_INF("uart_callback_set error, %d\r\n", ret);
		return 1;
	}


	//Start receiving by calling uart_rx_enable() and pass it the address of the receive  buffer 
	ret = uart_rx_enable(uart ,rx_buf,sizeof rx_buf,RECEIVE_TIMEOUT);
	if (ret) {
		return 1;
	}
	
	while (1) {
		gpio_pin_toggle_dt(&led0);
		k_msleep(SLEEP_TIME_MS);
	}

}